#!/usr/bin/perl -w

##################################################################
# transfer.pl
# This script transfers content to the modUpload module of
# the servX HTTP Server.
# The content may be a single file or a directory. Directories
# can be transfered recursively, which allows to transfer all 
# files within a directory and it's subdirectories at once.
#
# Last Modification:
#   $Author: bilsen $
#   $Date: 2009-10-30 12:55:29 +0100 (Fr, 30 Okt 2009) $
#   $Revision: 5670 $
#
##################################################################



use strict;
use LWP;

##################################################################
# DEFAULT PARAMETERS

# this is the source directory that has to be uploaded
my $strSrcRoot = "./Root";

# this is the target directory at the server
my $strTrgRoot = "";

# ignore *.svn files and directories by default
my $strFilter = "/.svn\$/";

# server address is localhost
my $strServer = '127.0.0.1';

# server path to post the request to
my $strServerPath = '/upload';

# default query string
my $strQuery = 'cmd=store&fullpage=false';

# directories are not transfered recursively by default
my $fRecursive = 0;

# basic authentication "username,password"
my $strAuth = '';
my $strDelim = ':';



##################################################################
# HANDLE SCRIPT ARGUMENTS

my $arg = shift;

while( $arg ) {
  #print("$arg\n");

  if( $arg eq '-help' or $arg eq '-?' ) {
    print("\nusage: transfer.pl [-r] [-s sourceRoot] [-t targetRoot] [-a[delimiter] realm:user:pass] [server]\n");
    print("\nOptions:\n");
    print("-r  Process source directories recursively\n");
	print("-s  'sourceRoot' defines the file or root source directory to transfer\n");
	print("-t  'targetRoot' defines root target directory to store the files in\n");
	print("-a  Basic authentication settings in the form 'realm:user:pass'\n");
	print("      a different delimiter than ':' may be defined by appending\n");
	print("      it to the -a option (e.g. \"-a* realm*user*pass\")\n");
	exit;
  }
  if( $arg eq '-s' ) { $strSrcRoot = shift; }
  elsif( $arg eq '-t' ) { $strTrgRoot = shift; }
  elsif( substr($arg,0,2) eq '-a' ) {    
    if( length($arg) > 2 ) { $strDelim = substr($arg,2); }
	$strAuth = shift;
  }
  elsif( $arg eq '-r' ) { $fRecursive = 1; }
  elsif( substr($arg,0,1) eq '-' ) {
    print("ERROR: Invalid option: $arg!\nUse -? for help\n");
	exit;
  }
  else { $strServer = $arg; }
  
  $arg = shift;
}


##################################################################
# MAIN

# welcome message
print("\ntransfer.pl/0.97\nUpload Tool for servX HTTP Server\n");

# globals

my $url = 'http://'.$strServer;
if( $strServerPath ne "" ) {
  $url = $url.$strServerPath;
}

if( $strQuery ne "" ) {
  $url = $url."?".$strQuery;
}

print("Target URL is $url\n\n");

# init HTTP client
my $client = LWP::UserAgent->new;
$client->agent("servX Upload Tool/0.97.1");

if( $strAuth ne '' ) {
  my @auth = split($strDelim, $strAuth);
  
  #print $auth[0]."\n";
  #print $auth[1]."\n";
  #print $auth[2]."\n";
  #print $strServer.':80'."\n";
  
  $client->credentials(
  $strServer.':80',
  $auth[0],
  $auth[1] => $auth[2],
  );
}



# global file counter
my %result = ( 'total' => 0, 'pass' => 0 );

if ( -d $strSrcRoot ) {
  if( $fRecursive ) {
    transferContentFromDir($strSrcRoot, $strTrgRoot);
  }
} else {
  if( -f $strSrcRoot ) {
    transferFile($strSrcRoot, $strTrgRoot);
  } else {
    print("ERROR: Cannot find $strSrcRoot\n");
	exit;
  }
}

printf("%d of %d files passed.\n", $result{pass}, $result{total} );

if( $result{total} == 0 ) {
  print("\nNo file found to transfer (forgot -r option?)\n\n");
} else {
  if( $result{pass} ==  $result{total} ) {
    print("\nTRANSFER SUCCEEDED!\n\n");
  } else {
    print("\nTRANSFER FAILED!\n\n");
  }
}


##################################################################
# FUNCTIONS

sub transferContentFromDir {
  my $dir = shift;
  my $trg = shift;

  print("Processing directory $dir\n");
  opendir(DIR, $dir) or die "cannot open directory";
  
  # get all directory entries
  my @files = grep(!/^\./, readdir(DIR));
  closedir(DIR);

  foreach (@files) {
    my $srcpath = "$dir/$_";
  
    # is this file a directory?
    if (-d $srcpath) {
      # transfer this directory recursively
      transferContentFromDir($srcpath, "$trg/$_");
    } else {
      # we have a file here
      transferFile($srcpath, $trg);
	  
    }
  }
}

sub transferFile {
  my $strFile = shift;
  my $strTrgPath = shift;
  
  $result{total}++;
  print("Transferring $strFile to /$strTrgPath ... ");
  my $response = $client->post(
    $url,
    [ 'file' => [$strFile],
      'path' => $strTrgPath,
    ],
    'Content_Type' => 'form-data',
  );

  if ($response->is_success()) {
    if( $response->content() =~ /success/ ) {
      $result{pass}++;
      print("ok\n");
	} else {
	  printf("ERROR: %s\n", $response->content());
	}
  } else {
    printf("ERROR: %s\n", $response->status_line());
  }
}

